
# This file was generated by 'versioneer.py' (0.28) from
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

import json

version_json = '''
{
 "date": "2023-05-02T03:32:19-0300",
 "dirty": false,
 "error": null,
 "full-revisionid": "df370b6aa09b16e0de4f66e8e28e0d9288cf3098",
 "version": "0.1.3"
}
'''  # END VERSION_JSON


def get_versions():
    return json.loads(version_json)
